library(logistf)
args <- commandArgs(trailingOnly = TRUE)

is_binary <- function(x){
  identical(as.vector(x),as.numeric(as.logical(x)))
}

# given gene_table, create list of individuals and alt dosage
get_counts <- function(x){
  counts <- c()
  inds <- c()
  for(i in x){
    a <- strsplit(i,":")
    counts <- append(counts,as.numeric(a[[1]][2]))
    inds <- append(inds,a[[1]][1])
  }
  return(list(counts,inds))
}

make_z_mat <- function(gene_table,all_inds){
  n_samples <- length(all_inds)
  ind_counts <- lapply(strsplit(as.character(gene_table$carriers),","),get_counts)
  n_variants <- nrow(gene_table)
  z_mat <- matrix(0,
                  nrow = n_samples,
                  ncol = n_variants)
  for(i in 1:length(ind_counts)){
    nonref_inds <- ind_counts[[i]][[2]]
    nonref_genos <- ind_counts[[i]][[1]]
    missing_field <- gene_table$missing[i]
    if(!is.na(gene_table$missing[i])){
      missing_inds <- strsplit(missing_field,",")[[1]]
      ind <- which(all_inds %in% missing_inds)
      if(length(ind) != 0){
        z_mat[ind,i] <- NA
      }
    }
    for(k in 1:length(nonref_inds)){
      geno <- nonref_genos[k]
      ind <- which(all_inds == nonref_inds[k])
      if(length(ind) != 0){
        z_mat[ind,i] <- geno
      }
    }
  }
  return(z_mat)
}

print("Readling all_binary")

# read derived, all_ICD10, all_medication, all_operation, all_self_cancer, all_self_noncancer
# merge together
all_phenos <- read.table("/MYPATH/ukbb_baseline_phenotypes.csv.gz", header = T, sep = ",")

# filter out unrelated/non-British inds
phenos <- all_phenos[which(all_phenos$UNREL == 1),]
phenos <- phenos[which(phenos$BRITISH == 1),]
phenos <- phenos[-which(phenos$sampleId == ""),]

# exclude phenotypes with <100 cases
exclude_i <- c()
for(i in 1:ncol(phenos)){
  if(is_binary(phenos[,i])){
    if(grepl("DEATH",names(phenos)[i])){
      # for death phenotypes, use 10, else 100
      if(sum(phenos[,i],na.rm=T) < 10){
        exclude_i <- append(exclude_i,i)
      }
      next()
    }
    if(sum(phenos[,i],na.rm=T) < 100){
      exclude_i <- append(exclude_i,i)
    }
  }
}
phenos <- phenos[,-exclude_i]
all_inds <- as.character(phenos$sampleId)

gene <- as.character(args[1])
print("Reading carrier tables")
big_lof_table <- read.table("/MYPATH/UKB.HC_LOF_canonical.txt.gz",
                            header = T,
                            colClasses = c("character","character","numeric",rep("character",8),"numeric","character","character"))

big_lof_table <- big_lof_table[union(which(big_lof_table$alt_af < 0.01),which(big_lof_table$alt_af > 0.99)),]
missing_lofs <- which(is.na(big_lof_table$carriers))
if(length(missing_lofs) > 0){
  big_lof_table <- big_lof_table[-missing_lofs,]
}

gene_lof_table <- big_lof_table[which(big_lof_table$gene == gene),]
z_mat <- make_z_mat(gene_lof_table,all_inds)
x <- as.numeric(rowSums(z_mat,na.rm=T))
covar_mat <- as.matrix(data.frame(phenos$AGE,phenos$SEX,phenos$PC1,phenos$PC2,phenos$PC3,phenos$PC4,phenos$PC5,phenos$PC6,phenos$PC7,phenos$PC8,phenos$PC9,phenos$PC10))
covar_mat_nosex <- as.matrix(data.frame(phenos$AGE,phenos$PC1,phenos$PC2,phenos$PC3,phenos$PC4,phenos$PC5,phenos$PC6,phenos$PC7,phenos$PC8,phenos$PC9,phenos$PC10))

n_carriers <- length(which(x > 0))
n_variants <- length(which(colSums(z_mat) > 0))

# get bl phenos
phewas_res <- data.frame()
bl_phenos <- c("IND","YEAR_OF_BIRTH","AGE","ADOPTED","BRITISH","UNREL","SEX","PC1","PC2","PC3","PC4","PC5","PC6","PC7","PC8","PC9","PC10","PC11","PC12","PC13","PC14","PC15","PC16","PC17","PC18","PC19","PC20","PC21","PC22","PC23","PC24","PC25","PC26","PC27","PC28","PC29","PC30","PC31","PC32","PC33","PC34","PC35","PC36","PC37","PC38","PC39","PC40","sampleId","HOME_AREA")

phewas_key <- read.csv("/MYPATH/phewas_pheno_key.csv",
                       header = F)
names(phewas_key) <- c("pheno","long_pheno")

for(i in 1:ncol(phenos)){
  pheno_name <- names(phenos)[i]
  print(pheno_name)
  long_pheno <- phewas_key$long_pheno[which(phewas_key$pheno == pheno_name)]
  if(pheno_name %in% bl_phenos){
    next()
  }
  y <- phenos[,i]
  # check if sex-specific
  # if pheno in one sex is less than 5% the number in other sex, make it sex specific
  pheno_male <- y[which(phenos$SEX == 1)]
  pheno_female <- y[which(phenos$SEX == 0)]
  male_specific <- FALSE
  female_specific <- FALSE
  if(length(which(!is.na(pheno_male))) < 0.05*length(which(!is.na(pheno_female)))){
    female_specific <- TRUE
  }
  if(length(which(!is.na(pheno_female))) < 0.05*length(which(!is.na(pheno_male)))){
    male_specific <- TRUE
  }
  if(is_binary(y)){
    pheno_type <- "binary"
    n_cases <- length(which(y == 1))
    n_controls <- length(which(y == 0))
    if(n_cases < 100 | n_controls < 100){
      next()
    }
    # both sexes
    test_type <- "logistic_regression"
    lreg <- glm(y~x+covar_mat,family = "binomial")
    if(!"x" %in% rownames(summary(lreg)$coefficients)){
      next()
    }
    beta <- summary(lreg)$coefficients["x",1]
    p <- summary(lreg)$coefficients["x",4]
    if(p < 0.05){
      test_type <- "firth_logistic_regression"
      lreg <- logistf(y~x+covar_mat)
      beta <- lreg$coefficients["x"]
      p <- lreg$prob["x"]
      if(p == 0){
        lreg <- glm(y~x+covar_mat,family = "binomial")
        p <- summary(lreg)$coefficients["x",4]
      }
    }
    # males
    test_type_males <- "logistic_regression"
    beta_males <- NA
    p_males <- NA
    if(!female_specific){
      lreg <- glm(y[which(phenos$SEX == 1)]~x[which(phenos$SEX == 1)]+covar_mat_nosex[which(phenos$SEX == 1),],
                  family = "binomial")
      if("x[which(phenos$SEX == 1)]" %in% rownames(summary(lreg)$coefficients)){
        beta_males <- summary(lreg)$coefficients["x[which(phenos$SEX == 1)]",1]
        p_males <- summary(lreg)$coefficients["x[which(phenos$SEX == 1)]",4]
      }
      if(!is.na(p_males) && p_males < 0.05){
        test_type_males <- "firth_logistic_regression"
        lreg <- logistf(y[which(phenos$SEX == 1)]~x[which(phenos$SEX == 1)]+covar_mat_nosex[which(phenos$SEX == 1),])
        beta_males <- lreg$coefficients["x[which(phenos$SEX == 1)]"]
        p_males <- lreg$prob["x[which(phenos$SEX == 1)]"]
        if(p_males == 0){
          lreg <- glm(y[which(phenos$SEX == 1)]~x[which(phenos$SEX == 1)]+covar_mat_nosex[which(phenos$SEX == 1),],
                      family = "binomial")
          p_males <- summary(lreg)$coefficients["x[which(phenos$SEX == 1)]",4]
        }
      }
    }
    # females
    test_type_females <- "logistic_regression"
    beta_females <- NA
    p_females <- NA
    if(!male_specific){
      lreg <- glm(y[which(phenos$SEX == 0)]~x[which(phenos$SEX == 0)]+covar_mat_nosex[which(phenos$SEX == 0),],
                  family = "binomial")
      if("x[which(phenos$SEX == 0)]" %in% rownames(summary(lreg)$coefficients)){
        beta_females <- summary(lreg)$coefficients["x[which(phenos$SEX == 0)]",1]
        p_females <- summary(lreg)$coefficients["x[which(phenos$SEX == 0)]",4]
      }
      if(!is.na(p_females) && p_females < 0.05){
        test_type_females <- "firth_logistic_regression"
        lreg <- logistf(y[which(phenos$SEX == 0)]~x[which(phenos$SEX == 0)]+covar_mat_nosex[which(phenos$SEX == 0),])
        beta_females <- lreg$coefficients["x[which(phenos$SEX == 0)]"]
        p_females <- lreg$prob["x[which(phenos$SEX == 0)]"]
        if(p_females == 0){
          lreg <- glm(y[which(phenos$SEX == 0)]~x[which(phenos$SEX == 0)]+covar_mat_nosex[which(phenos$SEX == 0),],
                      family = "binomial")
          p_females <- summary(lreg)$coefficients["x[which(phenos$SEX == 0)]",4]
        }
      }
    }
  }
  if(!is_binary(y)){
    # not binary. do linear regression
    pheno_type <- "continuous"
    test_type <- "linear_regression"
    test_type_males <- "linear_regression"
    test_type_females <- "linear_regression"
    # remove outliers - 5 SDs
    sdy <- sd(y,na.rm=T)
    meany <- mean(y,na.rm=T)
    outliers_i <- union(which(y > meany + 5*sdy),which(y < meany - 5*sdy))
    if(length(outliers_i) > 0){
      y[outliers_i] <- NA
    }
    n_cases <- length(which(!is.na(y)))
    if(n_cases < 100){
      next()
    }
    y_int <- qnorm((rank(y,na.last="keep")-0.5)/sum(!is.na(y)))
    n_controls <- NA
    n_values <- length(unique(y))
    if(n_values <= 12){
      next()
    }
    # both sexes
    beta <- NA
    p <- NA
    lreg <- lm(y~x+covar_mat)
    lreg_int <- lm(y_int~x+covar_mat)
    if(nrow(summary(lreg)$coefficients) == (ncol(covar_mat_nosex) + 3)){
      beta <- summary(lreg)$coefficients["x",1]
      p <- summary(lreg)$coefficients["x",4]
      beta_int <- summary(lreg_int)$coefficients["x",1]
      p_int <- summary(lreg_int)$coefficients["x",4]
    }
    # males
    if(female_specific){
      beta_males <- NA
      p_males <- NA
    }else{
      lreg <- lm(y[which(phenos$SEX == 1)]~x[which(phenos$SEX == 1)]+covar_mat_nosex[which(phenos$SEX == 1),])
      lreg_int <- lm(y_int[which(phenos$SEX == 1)]~x[which(phenos$SEX == 1)]+covar_mat_nosex[which(phenos$SEX == 1),])
      if(nrow(summary(lreg)$coefficients) == (ncol(covar_mat_nosex) + 2)){
        beta_males <- summary(lreg)$coefficients["x[which(phenos$SEX == 1)]",1]
        p_males <- summary(lreg)$coefficients["x[which(phenos$SEX == 1)]",4]
        beta_males_int <- summary(lreg_int)$coefficients["x[which(phenos$SEX == 1)]",1]
        p_males_int <- summary(lreg_int)$coefficients["x[which(phenos$SEX == 1)]",4]
      }
    }
    # females
    if(male_specific){
      beta_females <- NA
      p_females <- NA
    }else{
      lreg <- lm(y[which(phenos$SEX == 0)]~x[which(phenos$SEX == 0)]+covar_mat_nosex[which(phenos$SEX == 0),])
      lreg_int <- lm(y_int[which(phenos$SEX == 0)]~x[which(phenos$SEX == 0)]+covar_mat_nosex[which(phenos$SEX == 0),])
      if(nrow(summary(lreg)$coefficients) == (ncol(covar_mat_nosex) + 2)){
        beta_females <- summary(lreg)$coefficients["x[which(phenos$SEX == 0)]",1]
        p_females <- summary(lreg)$coefficients["x[which(phenos$SEX == 0)]",4]
        beta_females_int <- summary(lreg_int)$coefficients["x[which(phenos$SEX == 0)]",1]
        p_females_int <- summary(lreg_int)$coefficients["x[which(phenos$SEX == 0)]",4]
      }
    }
    res <- data.frame(gene,n_carriers,n_variants,long_pheno,paste(pheno_name,"_INT",sep=""),n_cases,n_controls,beta_int,p_int,test_type,beta_males_int,p_males_int,test_type_males,beta_females_int,p_females_int,test_type_females)
    names(res) <- c("gene","n_carriers","n_variants","long_pheno","pheno","n_cases","n_controls","beta","p","test_type","beta_males","p_males","test_type_males","beta_females","p_females","test_type_females")
    phewas_res <- rbind(phewas_res,res)
  }
  res <- data.frame(gene,n_carriers,n_variants,long_pheno,pheno_name,n_cases,n_controls,beta,p,test_type,beta_males,p_males,test_type_males,beta_females,p_females,test_type_females)
  names(res) <- c("gene","n_carriers","n_variants","long_pheno","pheno","n_cases","n_controls","beta","p","test_type","beta_males","p_males","test_type_males","beta_females","p_females","test_type_females")
  phewas_res <- rbind(phewas_res,res)
  if(!is.na(p)){
    if(p < 5e-4){
      print(paste(pheno_name,beta,p))
    }
  }
}


write.csv(phewas_res,paste("/MYPATH/phewas.maf00001.",gene,".v1.csv",sep=""), row.names = F)
