library(dplyr)
library(data.table)

#===========================================================
# read in DDD gene sets - DDG2P
#===========================================================
# The Development Disorder Genotype - Phenotype Database (DDG2P) is a curated list of genes reported to be associated with developmental disorders, compiled by clinicians as part of the DDD study to facilitate clinical feedback of likely causal variants. The DDG2P is categorised into the level of certainty that the gene causes developmental disease (e.g. Confirmed, Probable), the consequence of a mutation (e.g. Loss of Function, Activating) and the allelic status associated with disease (e.g. Monoallelic, Biallelic). Please note, genes with G2P category “Child IF” or confidence level “Possible” are not displayed in DECIPHER. Further information regarding G2P terminology can be viewed here: https://www.ebi.ac.uk/gene2phenotype/terminology

# https://decipher.sanger.ac.uk/ddd/ddgenes
# 12/23/2020
#===========================================================
allgene.file <- read.table("/MYPATH/UKB_gene_pos.txt",header=TRUE,stringsAsFactors=FALSE)
ddg2p.file <- read.csv("DDG2P_23_12_2020.csv")
allgene <- allgene.file$GENE
ddg2p_highconf <- ddg2p.file$gene.symbol[which(ddg2p.file$DDD.category=="confirmed" | ddg2p.file$DDD.category=="probable")]
ddg2p_all <- ddg2p.file$gene.symbol
non_ddg2p_all <- setdiff(allgene,ddg2p_all)

length(ddg2p_highconf)
length(ddg2p_all)
length(non_ddg2p_all)

#===========================================================
# read in pLI
#===========================================================
annot_table <- read.table("/MYPATH/gnomad.v2.1.1.lof_metrics.by_gene.txt.bgz",header=TRUE,stringsAsFactors=FALSE,sep="\t")
annot_table$pLI_damaging <- cut(annot_table$pLI, breaks=c(0,0.9,1), include.lowest=TRUE)
levels(annot_table$pLI_damaging) <- c("low","high")
annot_table$oe_lof_upper_damaging <- cut(annot_table$oe_lof_upper, breaks=c(0,0.35,2), include.lowest=TRUE)
levels(annot_table$oe_lof_upper_damaging) <- c("high","low")
pLI_intol <- annot_table$gene[which(annot_table$pLI_damaging=="high")]
pLI_tol <- annot_table$gene[which(annot_table$pLI_damaging=="low")]

#=======================================
# Gene lists
#=======================================
# pLI_intol
# pLI_tol
# ddg2p_highconf
# ddg2p_all

length(intersect(pLI_intol, ddg2p_highconf)) # 547
length(intersect(pLI_intol, ddg2p_all)) # 659
length(intersect(pLI_intol, non_ddg2p_all)) # 2290
length(intersect(pLI_tol, ddg2p_highconf)) # 1166
length(intersect(pLI_tol, ddg2p_all)) # 1378
length(intersect(pLI_tol, non_ddg2p_all)) # 13299

#=======================================
# Missense burden 
#=======================================
maflist <- c("00001")
annotlist <- c("MPC1_3","MPC1_2", "MPC1_0")
pathwaylist <- c("ddg2p_highconf", "ddg2p_all", "non_ddg2p_all")

for (maf in maflist){
	for (annot in annotlist){
		for (pathway in pathwaylist){
			filelist <- list.files("/MYPATH")
			searchterm <- paste0("maf",maf,".",annot)
			filelist <- filelist[is.na(stringr::str_extract(filelist, searchterm))==FALSE]
			filelist_burden <- filelist[is.na(stringr::str_extract(filelist, '\\b(info)\\b'))==TRUE]
			filelist_info <- filelist[is.na(stringr::str_extract(filelist, '\\b(info)\\b'))==FALSE]

			output_raw <- read.table(paste0("/MYPATH/",filelist_burden[1]), header=TRUE, stringsAsFactors=FALSE)["eid_sample"]
			output_gw_raw <- output_raw
			output_gw_raw.count <- 0
			output_pLI_intol_raw <- output_raw
			output_pLI_intol_raw.count <- 0
			output_pLI_tol_raw <- output_raw
			output_pLI_tol_raw.count <- 0

			for (i in 1:length(filelist_burden)){
				tmpfilename <- filelist_burden[i]
				tmpdat_raw <- as.data.frame(fread(paste0("/MYPATH/",tmpfilename), header=TRUE, stringsAsFactors=FALSE))

				tmpdat <- as.matrix(tmpdat_raw[,which(names(tmpdat_raw) %in% get(pathway))])
				print(c("tmpdat",dim(tmpdat)))
				if (dim(tmpdat)[2] > 0){
					output_gw_raw <- cbind.data.frame(output_gw_raw, rowSums(tmpdat,na.rm=TRUE))
					colnames(output_gw_raw)[length(names(output_gw_raw))] <- tmpfilename
					output_gw_raw.count <- output_gw_raw.count + dim(tmpdat)[2]
				}
				tmpdat <- as.matrix(tmpdat_raw[,which(names(tmpdat_raw) %in% get(pathway) & names(tmpdat_raw) %in% pLI_intol)])
				print(c("tmpdat",dim(tmpdat)))
				if (dim(tmpdat)[2] > 0){
					output_pLI_intol_raw <- cbind.data.frame(output_pLI_intol_raw, rowSums(tmpdat,na.rm=TRUE))
					colnames(output_pLI_intol_raw)[length(names(output_pLI_intol_raw))] <- tmpfilename
					output_pLI_intol_raw.count <- output_pLI_intol_raw.count + dim(tmpdat)[2]
				}
				tmpdat <- as.matrix(tmpdat_raw[,which(names(tmpdat_raw) %in% get(pathway) & names(tmpdat_raw) %in% pLI_tol)])
				print(c("tmpdat",dim(tmpdat)))
				if (dim(tmpdat)[2] > 0){
					output_pLI_tol_raw <- cbind.data.frame(output_pLI_tol_raw, rowSums(tmpdat,na.rm=TRUE))
					colnames(output_pLI_tol_raw)[length(names(output_pLI_tol_raw))] <- tmpfilename
					output_pLI_tol_raw.count <- output_pLI_tol_raw.count + dim(tmpdat)[2]
				}
			}
			output_gw_raw$burdensum        <- rowSums(output_gw_raw[,-which(names(output_gw_raw) %in% c("eid_sample"))])
			output_pLI_intol_raw$burdensum <- rowSums(output_pLI_intol_raw[,-which(names(output_pLI_intol_raw) %in% c("eid_sample"))])
			output_pLI_tol_raw$burdensum   <- rowSums(output_pLI_tol_raw[,-which(names(output_pLI_tol_raw) %in% c("eid_sample"))])
			output_gw <- output_gw_raw[,c("eid_sample","burdensum")]
			output_pLI_intol <- output_pLI_intol_raw[,c("eid_sample","burdensum")]
			output_pLI_tol <- output_pLI_tol_raw[,c("eid_sample","burdensum")]

			write.table(output_gw, paste0("ukbb.gw.maf",maf,".N",output_gw_raw.count,".missensesum_",annot,".",pathway,".txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
			write.table(output_pLI_intol, paste0("ukbb.pLI_intol.maf",maf,".N",output_pLI_intol_raw.count,".missensesum_",annot,".",pathway,".txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
			write.table(output_pLI_tol, paste0("ukbb.pLI_tol.maf",maf,".N",output_pLI_tol_raw.count,".missensesum_",annot,".",pathway,".txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
		}
	}
}


#=======================================
# Synonymous burden 
#=======================================
maflist <- c("00001")
pathwaylist <- c("ddg2p_highconf", "ddg2p_all", "non_ddg2p_all")

for (maf in maflist){
	for (pathway in pathwaylist){
		filelist <- list.files("/MYPATH")
		searchterm <- paste0("maf",maf)
		filelist <- filelist[is.na(stringr::str_extract(filelist, searchterm))==FALSE & is.na(stringr::str_extract(filelist, "chrX"))==TRUE]
		filelist_burden <- filelist[is.na(stringr::str_extract(filelist, '\\b(info)\\b'))==TRUE]
		filelist_info <- filelist[is.na(stringr::str_extract(filelist, '\\b(info)\\b'))==FALSE]
		output_raw <- read.table(paste0("/MYPATH/","ukbb_456k_v2_chr21.maf00001.synonymous.tsv"), header=TRUE, stringsAsFactors=FALSE)["eid_sample"]

		output_raw <- read.table(paste0("/MYPATH/",filelist_burden[1]), header=TRUE, stringsAsFactors=FALSE)["eid_sample"]
		output_gw_raw <- output_raw
		output_gw_raw.count <- 0
		output_pLI_intol_raw <- output_raw
		output_pLI_intol_raw.count <- 0
		output_pLI_tol_raw <- output_raw
		output_pLI_tol_raw.count <- 0

		for (i in 1:length(filelist_burden)){
			tmpfilename <- filelist_burden[i]
			tmpdat_raw <- as.data.frame(fread(paste0("/MYPATH/",tmpfilename), header=TRUE, stringsAsFactors=FALSE))

			tmpdat <- as.matrix(tmpdat_raw[,which(names(tmpdat_raw) %in% get(pathway))])
			print(c("tmpdat",dim(tmpdat)))
			if (dim(tmpdat)[2] > 0){
				output_gw_raw <- cbind.data.frame(output_gw_raw, rowSums(tmpdat,na.rm=TRUE))
				colnames(output_gw_raw)[length(names(output_gw_raw))] <- tmpfilename
				output_gw_raw.count <- output_gw_raw.count + dim(tmpdat)[2]
			}
			tmpdat <- as.matrix(tmpdat_raw[,which(names(tmpdat_raw) %in% get(pathway) & names(tmpdat_raw) %in% pLI_intol)])
			print(c("tmpdat",dim(tmpdat)))
			if (dim(tmpdat)[2] > 0){
				output_pLI_intol_raw <- cbind.data.frame(output_pLI_intol_raw, rowSums(tmpdat,na.rm=TRUE))
				colnames(output_pLI_intol_raw)[length(names(output_pLI_intol_raw))] <- tmpfilename
				output_pLI_intol_raw.count <- output_pLI_intol_raw.count + dim(tmpdat)[2]
			}
			tmpdat <- as.matrix(tmpdat_raw[,which(names(tmpdat_raw) %in% get(pathway) & names(tmpdat_raw) %in% pLI_tol)])
			print(c("tmpdat",dim(tmpdat)))
			if (dim(tmpdat)[2] > 0){
				output_pLI_tol_raw <- cbind.data.frame(output_pLI_tol_raw, rowSums(tmpdat,na.rm=TRUE))
				colnames(output_pLI_tol_raw)[length(names(output_pLI_tol_raw))] <- tmpfilename
				output_pLI_tol_raw.count <- output_pLI_tol_raw.count + dim(tmpdat)[2]
			}
		}
		output_gw_raw$burdensum        <- rowSums(output_gw_raw[,-which(names(output_gw_raw) %in% c("eid_sample"))])
		output_pLI_intol_raw$burdensum <- rowSums(output_pLI_intol_raw[,-which(names(output_pLI_intol_raw) %in% c("eid_sample"))])
		output_pLI_tol_raw$burdensum   <- rowSums(output_pLI_tol_raw[,-which(names(output_pLI_tol_raw) %in% c("eid_sample"))])
		output_gw <- output_gw_raw[,c("eid_sample","burdensum")]
		output_pLI_intol <- output_pLI_intol_raw[,c("eid_sample","burdensum")]
		output_pLI_tol <- output_pLI_tol_raw[,c("eid_sample","burdensum")]

		write.table(output_gw, paste0("ukbb.gw.maf",maf,".N",output_gw_raw.count,".synonymoussum.",pathway,".txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
		write.table(output_pLI_intol, paste0("ukbb.pLI_intol.maf",maf,".N",output_pLI_intol_raw.count,".synonymoussum.",pathway,".txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
		write.table(output_pLI_tol, paste0("ukbb.pLI_tol.maf",maf,".N",output_pLI_tol_raw.count,".synonymoussum.",pathway,".txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
	}
}


#=======================================
# PTV burden 
#=======================================
maflist <- c("00001")
pathwaylist <- c("ddg2p_highconf", "ddg2p_all", "non_ddg2p_all")

for (maf in maflist){
	for (pathway in pathwaylist){
		filelist <- list.files("/MYPATH")
		searchterm <- paste0("maf",maf)
		filelist <- filelist[is.na(stringr::str_extract(filelist, searchterm))==FALSE & is.na(stringr::str_extract(filelist, "chrX"))==TRUE]
		filelist_burden <- filelist[is.na(stringr::str_extract(filelist, '\\b(info)\\b'))==TRUE]
		filelist_info <- filelist[is.na(stringr::str_extract(filelist, '\\b(info)\\b'))==FALSE]
		output_raw <- read.table(paste0("/MYPATH/","ukbb_456k_v2_chunk156.maf00001.ptvburden.tsv"), header=TRUE, stringsAsFactors=FALSE)["eid_sample"]

		output_raw <- read.table(paste0("/MYPATH/",filelist_burden[1]), header=TRUE, stringsAsFactors=FALSE)["eid_sample"]
		output_gw_raw <- output_raw
		output_gw_raw.count <- 0
		output_pLI_intol_raw <- output_raw
		output_pLI_intol_raw.count <- 0
		output_pLI_tol_raw <- output_raw
		output_pLI_tol_raw.count <- 0

		for (i in 1:length(filelist_burden)){
			tmpfilename <- filelist_burden[i]
			tmpdat_raw <- as.data.frame(fread(paste0("/MYPATH/",tmpfilename), header=TRUE, stringsAsFactors=FALSE))

			tmpdat <- as.matrix(tmpdat_raw[,which(names(tmpdat_raw) %in% get(pathway))])
			print(c("tmpdat",dim(tmpdat)))
			if (dim(tmpdat)[2] > 0){
				output_gw_raw <- cbind.data.frame(output_gw_raw, rowSums(tmpdat,na.rm=TRUE))
				colnames(output_gw_raw)[length(names(output_gw_raw))] <- tmpfilename
				output_gw_raw.count <- output_gw_raw.count + dim(tmpdat)[2]
			}
			tmpdat <- as.matrix(tmpdat_raw[,which(names(tmpdat_raw) %in% get(pathway) & names(tmpdat_raw) %in% pLI_intol)])
			print(c("tmpdat",dim(tmpdat)))
			if (dim(tmpdat)[2] > 0){
				output_pLI_intol_raw <- cbind.data.frame(output_pLI_intol_raw, rowSums(tmpdat,na.rm=TRUE))
				colnames(output_pLI_intol_raw)[length(names(output_pLI_intol_raw))] <- tmpfilename
				output_pLI_intol_raw.count <- output_pLI_intol_raw.count + dim(tmpdat)[2]
			}
			tmpdat <- as.matrix(tmpdat_raw[,which(names(tmpdat_raw) %in% get(pathway) & names(tmpdat_raw) %in% pLI_tol)])
			print(c("tmpdat",dim(tmpdat)))
			if (dim(tmpdat)[2] > 0){
				output_pLI_tol_raw <- cbind.data.frame(output_pLI_tol_raw, rowSums(tmpdat,na.rm=TRUE))
				colnames(output_pLI_tol_raw)[length(names(output_pLI_tol_raw))] <- tmpfilename
				output_pLI_tol_raw.count <- output_pLI_tol_raw.count + dim(tmpdat)[2]
			}
		}
		output_gw_raw$burdensum        <- rowSums(output_gw_raw[,-which(names(output_gw_raw) %in% c("eid_sample"))])
		output_pLI_intol_raw$burdensum <- rowSums(output_pLI_intol_raw[,-which(names(output_pLI_intol_raw) %in% c("eid_sample"))])
		output_pLI_tol_raw$burdensum   <- rowSums(output_pLI_tol_raw[,-which(names(output_pLI_tol_raw) %in% c("eid_sample"))])
		output_gw <- output_gw_raw[,c("eid_sample","burdensum")]
		output_pLI_intol <- output_pLI_intol_raw[,c("eid_sample","burdensum")]
		output_pLI_tol <- output_pLI_tol_raw[,c("eid_sample","burdensum")]

		write.table(output_gw, paste0("ukbb.gw.maf",maf,".N",output_gw_raw.count,".ptvsum.",pathway,".txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
		write.table(output_pLI_intol, paste0("ukbb.pLI_intol.maf",maf,".N",output_pLI_intol_raw.count,".ptvsum.",pathway,".txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
		write.table(output_pLI_tol, paste0("ukbb.pLI_tol.maf",maf,".N",output_pLI_tol_raw.count,".ptvsum.",pathway,".txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
	}
}

