library(dplyr)
library(data.table)

#===========================================================
# read in pathways
#===========================================================
# Human brain atlas
# https://www.proteinatlas.org/humanproteome/brain/human+brain#regional_expression_within_the_brain
# 11/27/2020

# 								Regionally elevated	Low regional specificity	Missing regional classification	Total
# Elevated in brain				520					1776						291								2587
# Elevated in other tissue 		483					4295						520								5298
# but expressed in brain
# Low tissue specificity		56					8027						302								8385
# Total							1059				14098						1113							16270

# tissue_category_rna_brain_tissue.tsv # N = 2587
# tissue_category_rna_any_tissue.tsv # N = 5298
# tissue_category_rna_any_low.tsv # N = 8385

hba_brain.file <- fread("tissue_category_rna_brain_tissue.tsv")
hba_other.file <- fread("tissue_category_rna_any_tissue.tsv")
hba_nospe.file <- fread("tissue_category_rna_any_low.tsv")

hba_brain <- hba_brain.file$Gene
hba_other <- hba_other.file$Gene
hba_nospe <- hba_nospe.file$Gene

length(hba_brain)
length(hba_other)
length(hba_nospe)

#===========================================================
# read in pLI
#===========================================================
annot_table <- read.table("/MYPATH/gnomad.v2.1.1.lof_metrics.by_gene.txt.bgz",header=TRUE,stringsAsFactors=FALSE,sep="\t")
annot_table$pLI_damaging <- cut(annot_table$pLI, breaks=c(0,0.9,1), include.lowest=TRUE)
levels(annot_table$pLI_damaging) <- c("low","high")
annot_table$oe_lof_upper_damaging <- cut(annot_table$oe_lof_upper, breaks=c(0,0.35,2), include.lowest=TRUE)
levels(annot_table$oe_lof_upper_damaging) <- c("high","low")
pLI_intol <- annot_table$gene[which(annot_table$pLI_damaging=="high")]
pLI_tol <- annot_table$gene[which(annot_table$pLI_damaging=="low")]

#=======================================
# Gene lists
#=======================================
# pLI_intol
# pLI_tol
# hba_brain
# hba_other
# hba_nospe

length(intersect(pLI_intol, hba_brain)) # 527
length(intersect(pLI_intol, hba_other)) # 546
length(intersect(pLI_intol, hba_nospe)) # 1845
length(intersect(pLI_tol, hba_brain)) # 1854
length(intersect(pLI_tol, hba_other)) # 4351
length(intersect(pLI_tol, hba_nospe)) # 6047

#=======================================
# Missense burden 
#=======================================
maflist <- c("00001")
annotlist <- c("MPC1_3","MPC1_2", "MPC1_0")
pathwaylist <- c("hba_brain", "hba_other", "hba_nospe")

for (maf in maflist){
	for (annot in annotlist){
		for (pathway in pathwaylist){
			filelist <- list.files("/MYPATH/MYMISSENSEFILE")
			searchterm <- paste0("maf",maf,".",annot)
			filelist <- filelist[is.na(stringr::str_extract(filelist, searchterm))==FALSE]
			filelist_burden <- filelist[is.na(stringr::str_extract(filelist, '\\b(info)\\b'))==TRUE]
			filelist_info <- filelist[is.na(stringr::str_extract(filelist, '\\b(info)\\b'))==FALSE]

			output_raw <- read.table(paste0("/MYPATH/",filelist_burden[1]), header=TRUE, stringsAsFactors=FALSE)["eid_sample"]
			output_gw_raw <- output_raw
			output_gw_raw.count <- 0
			output_pLI_intol_raw <- output_raw
			output_pLI_intol_raw.count <- 0
			output_pLI_tol_raw <- output_raw
			output_pLI_tol_raw.count <- 0

			for (i in 1:length(filelist_burden)){
				tmpfilename <- filelist_burden[i]
				tmpdat_raw <- as.data.frame(fread(paste0("/MYPATH/",tmpfilename), header=TRUE, stringsAsFactors=FALSE))

				tmpdat <- as.matrix(tmpdat_raw[,which(names(tmpdat_raw) %in% get(pathway))])
				print(c("tmpdat",dim(tmpdat)))
				if (dim(tmpdat)[2] > 0){
					output_gw_raw <- cbind.data.frame(output_gw_raw, rowSums(tmpdat,na.rm=TRUE))
					colnames(output_gw_raw)[length(names(output_gw_raw))] <- tmpfilename
					output_gw_raw.count <- output_gw_raw.count + dim(tmpdat)[2]
				}
				tmpdat <- as.matrix(tmpdat_raw[,which(names(tmpdat_raw) %in% get(pathway) & names(tmpdat_raw) %in% pLI_intol)])
				print(c("tmpdat",dim(tmpdat)))
				if (dim(tmpdat)[2] > 0){
					output_pLI_intol_raw <- cbind.data.frame(output_pLI_intol_raw, rowSums(tmpdat,na.rm=TRUE))
					colnames(output_pLI_intol_raw)[length(names(output_pLI_intol_raw))] <- tmpfilename
					output_pLI_intol_raw.count <- output_pLI_intol_raw.count + dim(tmpdat)[2]
				}
				tmpdat <- as.matrix(tmpdat_raw[,which(names(tmpdat_raw) %in% get(pathway) & names(tmpdat_raw) %in% pLI_tol)])
				print(c("tmpdat",dim(tmpdat)))
				if (dim(tmpdat)[2] > 0){
					output_pLI_tol_raw <- cbind.data.frame(output_pLI_tol_raw, rowSums(tmpdat,na.rm=TRUE))
					colnames(output_pLI_tol_raw)[length(names(output_pLI_tol_raw))] <- tmpfilename
					output_pLI_tol_raw.count <- output_pLI_tol_raw.count + dim(tmpdat)[2]
				}
			}
			output_gw_raw$burdensum        <- rowSums(output_gw_raw[,-which(names(output_gw_raw) %in% c("eid_sample"))])
			output_pLI_intol_raw$burdensum <- rowSums(output_pLI_intol_raw[,-which(names(output_pLI_intol_raw) %in% c("eid_sample"))])
			output_pLI_tol_raw$burdensum   <- rowSums(output_pLI_tol_raw[,-which(names(output_pLI_tol_raw) %in% c("eid_sample"))])
			output_gw <- output_gw_raw[,c("eid_sample","burdensum")]
			output_pLI_intol <- output_pLI_intol_raw[,c("eid_sample","burdensum")]
			output_pLI_tol <- output_pLI_tol_raw[,c("eid_sample","burdensum")]

			write.table(output_gw, paste0("ukbb.gw.maf",maf,".N",output_gw_raw.count,".missensesum_",annot,".",pathway,".txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
			write.table(output_pLI_intol, paste0("ukbb.pLI_intol.maf",maf,".N",output_pLI_intol_raw.count,".missensesum_",annot,".",pathway,".txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
			write.table(output_pLI_tol, paste0("ukbb.pLI_tol.maf",maf,".N",output_pLI_tol_raw.count,".missensesum_",annot,".",pathway,".txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
		}
	}
}


#=======================================
# Synonymous burden 
#=======================================
maflist <- c("00001")
pathwaylist <- c("hba_brain", "hba_other", "hba_nospe")

for (maf in maflist){
	for (pathway in pathwaylist){
		filelist <- list.files("/MYPATH/MYSYNONYMOUSFILE")
		searchterm <- paste0("maf",maf)
		filelist <- filelist[is.na(stringr::str_extract(filelist, searchterm))==FALSE & is.na(stringr::str_extract(filelist, "chrX"))==TRUE]
		filelist_burden <- filelist[is.na(stringr::str_extract(filelist, '\\b(info)\\b'))==TRUE]
		filelist_info <- filelist[is.na(stringr::str_extract(filelist, '\\b(info)\\b'))==FALSE]
		output_raw <- read.table(paste0("/camhpc/dept/human_genetics/UKBB_exome_variant_450k/synonymous_456k_matrix_v1/","ukbb_456k_v2_chr21.maf00001.synonymous.tsv"), header=TRUE, stringsAsFactors=FALSE)["eid_sample"]

		output_raw <- read.table(paste0("/camhpc/dept/human_genetics/UKBB_exome_variant_450k/synonymous_456k_matrix_v1/",filelist_burden[1]), header=TRUE, stringsAsFactors=FALSE)["eid_sample"]
		output_gw_raw <- output_raw
		output_gw_raw.count <- 0
		output_pLI_intol_raw <- output_raw
		output_pLI_intol_raw.count <- 0
		output_pLI_tol_raw <- output_raw
		output_pLI_tol_raw.count <- 0

		for (i in 1:length(filelist_burden)){
			tmpfilename <- filelist_burden[i]
			tmpdat_raw <- as.data.frame(fread(paste0("/camhpc/dept/human_genetics/UKBB_exome_variant_450k/synonymous_456k_matrix_v1/",tmpfilename), header=TRUE, stringsAsFactors=FALSE))

			tmpdat <- as.matrix(tmpdat_raw[,which(names(tmpdat_raw) %in% get(pathway))])
			print(c("tmpdat",dim(tmpdat)))
			if (dim(tmpdat)[2] > 0){
				output_gw_raw <- cbind.data.frame(output_gw_raw, rowSums(tmpdat,na.rm=TRUE))
				colnames(output_gw_raw)[length(names(output_gw_raw))] <- tmpfilename
				output_gw_raw.count <- output_gw_raw.count + dim(tmpdat)[2]
			}
			tmpdat <- as.matrix(tmpdat_raw[,which(names(tmpdat_raw) %in% get(pathway) & names(tmpdat_raw) %in% pLI_intol)])
			print(c("tmpdat",dim(tmpdat)))
			if (dim(tmpdat)[2] > 0){
				output_pLI_intol_raw <- cbind.data.frame(output_pLI_intol_raw, rowSums(tmpdat,na.rm=TRUE))
				colnames(output_pLI_intol_raw)[length(names(output_pLI_intol_raw))] <- tmpfilename
				output_pLI_intol_raw.count <- output_pLI_intol_raw.count + dim(tmpdat)[2]
			}
			tmpdat <- as.matrix(tmpdat_raw[,which(names(tmpdat_raw) %in% get(pathway) & names(tmpdat_raw) %in% pLI_tol)])
			print(c("tmpdat",dim(tmpdat)))
			if (dim(tmpdat)[2] > 0){
				output_pLI_tol_raw <- cbind.data.frame(output_pLI_tol_raw, rowSums(tmpdat,na.rm=TRUE))
				colnames(output_pLI_tol_raw)[length(names(output_pLI_tol_raw))] <- tmpfilename
				output_pLI_tol_raw.count <- output_pLI_tol_raw.count + dim(tmpdat)[2]
			}
		}
		output_gw_raw$burdensum        <- rowSums(output_gw_raw[,-which(names(output_gw_raw) %in% c("eid_sample"))])
		output_pLI_intol_raw$burdensum <- rowSums(output_pLI_intol_raw[,-which(names(output_pLI_intol_raw) %in% c("eid_sample"))])
		output_pLI_tol_raw$burdensum   <- rowSums(output_pLI_tol_raw[,-which(names(output_pLI_tol_raw) %in% c("eid_sample"))])
		output_gw <- output_gw_raw[,c("eid_sample","burdensum")]
		output_pLI_intol <- output_pLI_intol_raw[,c("eid_sample","burdensum")]
		output_pLI_tol <- output_pLI_tol_raw[,c("eid_sample","burdensum")]

		write.table(output_gw, paste0("ukbb.gw.maf",maf,".N",output_gw_raw.count,".synonymoussum.",pathway,".txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
		write.table(output_pLI_intol, paste0("ukbb.pLI_intol.maf",maf,".N",output_pLI_intol_raw.count,".synonymoussum.",pathway,".txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
		write.table(output_pLI_tol, paste0("ukbb.pLI_tol.maf",maf,".N",output_pLI_tol_raw.count,".synonymoussum.",pathway,".txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
	}
}


#=======================================
# PTV burden 
#=======================================
maflist <- c("00001")
pathwaylist <- c("hba_brain", "hba_other", "hba_nospe")

for (maf in maflist){
	for (pathway in pathwaylist){
		filelist <- list.files("/MYPATH/MYPTVFILE")
		searchterm <- paste0("maf",maf)
		filelist <- filelist[is.na(stringr::str_extract(filelist, searchterm))==FALSE & is.na(stringr::str_extract(filelist, "chrX"))==TRUE]
		filelist_burden <- filelist[is.na(stringr::str_extract(filelist, '\\b(info)\\b'))==TRUE]
		filelist_info <- filelist[is.na(stringr::str_extract(filelist, '\\b(info)\\b'))==FALSE]
		output_raw <- read.table(paste0("/MYPATH/","ukbb_chunk1.maf00001.ptvburden.tsv"), header=TRUE, stringsAsFactors=FALSE)["eid_sample"]

		output_raw <- read.table(paste0("/MYPATH/",filelist_burden[1]), header=TRUE, stringsAsFactors=FALSE)["eid_sample"]
		output_gw_raw <- output_raw
		output_gw_raw.count <- 0
		output_pLI_intol_raw <- output_raw
		output_pLI_intol_raw.count <- 0
		output_pLI_tol_raw <- output_raw
		output_pLI_tol_raw.count <- 0

		for (i in 1:length(filelist_burden)){
			tmpfilename <- filelist_burden[i]
			tmpdat_raw <- as.data.frame(fread(paste0("/MYPATH/ptv_456k_v2/",tmpfilename), header=TRUE, stringsAsFactors=FALSE))

			tmpdat <- as.matrix(tmpdat_raw[,which(names(tmpdat_raw) %in% get(pathway))])
			print(c("tmpdat",dim(tmpdat)))
			if (dim(tmpdat)[2] > 0){
				output_gw_raw <- cbind.data.frame(output_gw_raw, rowSums(tmpdat,na.rm=TRUE))
				colnames(output_gw_raw)[length(names(output_gw_raw))] <- tmpfilename
				output_gw_raw.count <- output_gw_raw.count + dim(tmpdat)[2]
			}
			tmpdat <- as.matrix(tmpdat_raw[,which(names(tmpdat_raw) %in% get(pathway) & names(tmpdat_raw) %in% pLI_intol)])
			print(c("tmpdat",dim(tmpdat)))
			if (dim(tmpdat)[2] > 0){
				output_pLI_intol_raw <- cbind.data.frame(output_pLI_intol_raw, rowSums(tmpdat,na.rm=TRUE))
				colnames(output_pLI_intol_raw)[length(names(output_pLI_intol_raw))] <- tmpfilename
				output_pLI_intol_raw.count <- output_pLI_intol_raw.count + dim(tmpdat)[2]
			}
			tmpdat <- as.matrix(tmpdat_raw[,which(names(tmpdat_raw) %in% get(pathway) & names(tmpdat_raw) %in% pLI_tol)])
			print(c("tmpdat",dim(tmpdat)))
			if (dim(tmpdat)[2] > 0){
				output_pLI_tol_raw <- cbind.data.frame(output_pLI_tol_raw, rowSums(tmpdat,na.rm=TRUE))
				colnames(output_pLI_tol_raw)[length(names(output_pLI_tol_raw))] <- tmpfilename
				output_pLI_tol_raw.count <- output_pLI_tol_raw.count + dim(tmpdat)[2]
			}
		}
		output_gw_raw$burdensum        <- rowSums(output_gw_raw[,-which(names(output_gw_raw) %in% c("eid_sample"))])
		output_pLI_intol_raw$burdensum <- rowSums(output_pLI_intol_raw[,-which(names(output_pLI_intol_raw) %in% c("eid_sample"))])
		output_pLI_tol_raw$burdensum   <- rowSums(output_pLI_tol_raw[,-which(names(output_pLI_tol_raw) %in% c("eid_sample"))])
		output_gw <- output_gw_raw[,c("eid_sample","burdensum")]
		output_pLI_intol <- output_pLI_intol_raw[,c("eid_sample","burdensum")]
		output_pLI_tol <- output_pLI_tol_raw[,c("eid_sample","burdensum")]

		write.table(output_gw, paste0("ukbb.gw.maf",maf,".N",output_gw_raw.count,".ptvsum.",pathway,".txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
		write.table(output_pLI_intol, paste0("ukbb.pLI_intol.maf",maf,".N",output_pLI_intol_raw.count,".ptvsum.",pathway,".txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
		write.table(output_pLI_tol, paste0("ukbb.pLI_tol.maf",maf,".N",output_pLI_tol_raw.count,".ptvsum.",pathway,".txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
	}
}

