library(dplyr)
library(data.table)
library(GSA)

#===========================================================
# read in pathways
#===========================================================

############################################################
# MSigDB v7.2
# https://www.gsea-msigdb.org/gsea/msigdb
# 11/28/2020
#---------------------------------------
# C2: curated gene sets
#---------------------------------------
# Gene sets in this collection are curated from various sources, including online pathway databases and the biomedical literature. Many sets are also contributed by individual domain experts. The gene set page for each gene set lists its source. The C2 collection is divided into the following two sub-collections: Chemical and genetic perturbations (CGP) and Canonical pathways (CP).

# CP: Canonical pathways
# Gene sets from pathway databases. Usually, these gene sets are canonical representations of a biological process compiled by domain experts.

# 	BIOCARTA subset of CP
# 	Canonical Pathways gene sets derived from the BioCarta pathway database.

# 	KEGG subset of CP
# 	Canonical Pathways gene sets derived from the KEGG pathway database.

# 	PID subset of CP
# 	Canonical Pathways gene sets derived from the PID pathway database.

# 	REACTOME subset of CP
# 	Canonical Pathways gene sets derived from the Reactome pathway database.

# 	WikiPathways subset of CP
# 	Canonical Pathways gene sets derived from the WikiPathways pathway database.

#---------------------------------------
# C5: ontology gene sets	
#---------------------------------------
# Gene sets that contain genes annotated by the same ontology term. The C5 collection is divided into two sub-collections, the first derived from the Gene Ontology resource (GO) which contains BP, CC, and MF components and a second derived from the Human Phenotype Ontology (HPO). 
	
# GO: Gene Ontology gene sets
# All gene sets derived from Gene Ontology. 

	# BP: subset of GO
	# Gene sets derived from the GO Biological Process ontology. 

	# CC: subset of GO
	# Gene sets derived from the GO Cellular Component ontology. 

	# MF: subset of GO
	# Gene sets derived from the GO Molecular Function ontology. 
############################################################

msigdb_filelist <- c("c2.cp.kegg.v7.2.symbols.gmt", "c2.cp.wikipathways.v7.2.symbols.gmt", "c2.cp.biocarta.v7.2.symbols.gmt", "c2.cp.reactome.v7.2.symbols.gmt", "c2.cp.pid.v7.2.symbols.gmt", "c5.go.mf.v7.2.symbols.gmt", "c5.go.bp.v7.2.symbols.gmt", "c5.go.cc.v7.2.symbols.gmt")

for (msigdb_file in msigdb_filelist){
data <-  GSA.read.gmt(msigdb_file)
len_vec=c()           # Now create a vector for containing the length of genes at each position
len_vec[1] = 3
for(i in 1:length(data$genesets)){len_vec[i] <- c(length(data$genesets[[i]]))}
pathway_vec <- unlist(Vectorize(rep.int)(data$geneset.names, len_vec),use.names = FALSE) # Now create a vector for all the pathways in the data 
pathway_df <- as.data.frame(cbind(pathway_vec,unlist(data$genesets,use.names = FALSE))) # This gives your desired dataframe
colnames(pathway_df) <- c("PATHWAY", "GENE")
pathway_df$PATHWAY <- as.character(pathway_df$PATHWAY)
pathway_df$GENE <- as.character(pathway_df$GENE)
length(unique(pathway_df$GENE)) # 12396
pathwaylist <- unique(pathway_df$PATHWAY) 


#=======================================
# PTV burden 
#=======================================
maflist <- c("00001")

for (maf in maflist){
	gc()
	filelist <- list.files("/MYPATH/MYPTVFILE")
	searchterm <- paste0("maf",maf)
	filelist <- filelist[is.na(stringr::str_extract(filelist, searchterm))==FALSE & is.na(stringr::str_extract(filelist, "chrX"))==TRUE]
	filelist_burden <- filelist[is.na(stringr::str_extract(filelist, '\\b(info)\\b'))==TRUE]
	filelist_info <- filelist[is.na(stringr::str_extract(filelist, '\\b(info)\\b'))==FALSE]

	sampleidlist <- read.table(paste0("/MYPATH/",filelist_burden[1]), header=TRUE, stringsAsFactors=FALSE)["eid_sample"]
	output_matrix_raw <- as.data.frame(matrix(0,nrow=length(sampleidlist$eid_sample),ncol=length(pathwaylist)),stringsAsFactors=FALSE)
	colnames(output_matrix_raw) <- pathwaylist
	output_matrix <- cbind(sampleidlist, output_matrix_raw)

	output_count_raw <- as.data.frame(matrix(0,nrow=length(pathwaylist),ncol=1),stringsAsFactors=FALSE)
	output_count <- cbind(pathwaylist, output_count_raw)
	colnames(output_count) <- c("PATHWAY","N_GENE")
	output_count$PATHWAY <- as.character(output_count$PATHWAY)

	for (i in 1:length(filelist_burden)){
		tmpfilename <- filelist_burden[i]
		tmpdat_raw <- as.data.frame(fread(paste0("/MYPATH/",tmpfilename), header=TRUE, stringsAsFactors=FALSE))
		for (j in (1:length(pathwaylist))){
			pathway.genelist <- pathway_df[which(pathway_df$PATHWAY==pathwaylist[j]),"GENE"]
			tmpdat <- as.matrix(tmpdat_raw[,which(names(tmpdat_raw) %in% pathway.genelist)])
			if (dim(tmpdat)[2] > 0){
				output_matrix[,pathwaylist[j]] <- output_matrix[,pathwaylist[j]] + rowSums(tmpdat,na.rm=TRUE)
				output_count[j,"N_GENE"] <- output_count[j,"N_GENE"] + dim(tmpdat)[2]
			}
		gc()
		}
	}
	gc()
	write.table(output_matrix, paste0("ukbb.gw.maf",maf,".ptvsum.",paste0(paste0(strsplit(msigdb_file,'[.]')[[1]][1:3],collapse="_"),"_set",setnumber),".txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
	write.table(output_count, paste0("ukbb.gw.maf",maf,".ptvsum.",paste0(paste0(strsplit(msigdb_file,'[.]')[[1]][1:3],collapse="_"),"_set",setnumber),".N_GENE.txt"), col.names=TRUE, row.names=FALSE, quote=FALSE,sep=" ")
}
gc()
}
